// Your Name (username)

#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <stdlib.h>
#include <ctype.h>

// How many bytes a single food takes up in the file. Not the same as sizeof(Food)!
#define FILE_FOOD_LEN 32

// Returns whether two strings are equal. Nicer than strcmp() == 0.
bool streq(const char* a, const char* b) {
	return strcmp(a, b) == 0;
}

// returns 1 if the two strings are equal ignoring case, and 0 otherwise.
int streq_nocase(const char* a, const char* b) {
	for(; *a && *b; a++, b++) if(tolower(*a) != tolower(*b)) return 0;
	return *a == 0 && *b == 0;
}

// Called for fatal errors - prints the message and exits the program.
void fatal(const char* msg) {
	fprintf(stderr, "%s\n", msg);
	exit(1);
}

// ------------------------------------------------------------------------------------------------
// Department enum

// The departments in our store.
typedef enum {
	Invalid,

	Bakery,
	Canned,
	Frozen,
	Produce,
} Department;

// Converts a string to a Department, case-insensitively.
// Returns -1 if the string is not a department name.
Department str_to_department(const char* str) {
	if(streq_nocase(str, "bakery"))  return Bakery;
	if(streq_nocase(str, "canned"))  return Canned;
	if(streq_nocase(str, "frozen"))  return Frozen;
	if(streq_nocase(str, "produce")) return Produce;
	return Invalid;
}

// Converts a Department to a string.
// Returns NULL if the department is invalid.
const char* department_to_str(Department dept) {
	switch(dept) {
		case Bakery:  return "Bakery";
		case Canned:  return "Canned";
		case Frozen:  return "Frozen";
		case Produce: return "Produce";
		default:      return NULL;
	}
}

// ------------------------------------------------------------------------------------------------
// Food struct

// In-memory representation of a single food item.
typedef struct {
	Department dept;
	char name[16];
	double price;
	unsigned int stock;
} Food;

// Prints a single food item in a nice format.
void print_food(Food* f) {
	printf("[%-7s] %s ($%.2f each, %u in stock)\n",
		department_to_str(f->dept),
		f->name,
		f->price,
		f->stock);
}

// ------------------------------------------------------------------------------------------------
// Database

// attempts to create a new empty database. fails if the file already exists.
void new_database(const char* filename) {
	// 1. fopen the `filename` file with the "wbx" mode.
	//   - (x makes it so that it will NOT overwrite any existing file of the same name.)
	// 2. if fopen returned NULL, use the `fatal` function to print out the message:
	//    "could not create database file."
	// 3. otherwise, write the magic "FOOD" (4 characters exactly) and an int containing
	//    the value 1 to the file, then fclose it.
}

// opens `filename`, checks that it's a valid database (right magic and endianness),
// and returns the open FILE* if successful. Exits the program if not.
FILE* open_database(const char* filename) {
	// 1. fopen the `filename` file with the "rb+" mode (read and write binary).
	// 2. if fopen returned NULL, use the `fatal` function to print out the message:
	//    "could not open file."
	// 3. use fread to read the magic (4 characters exactly) and if it's not "FOOD", use `fatal`
	//    with the message "file doesn't appear to be a food database (bad magic)."
	// 4. use fread to read the endianness (an int). If it's 0, use `fatal` to say
	//    "big-endian databases are unsupported." if it's not 1, use `fatal` to say
	//    "file doesn't appear to be a food database (bad endianness).".
	// 5. use ftell to see where "here" is. then seek to the end of the file, use ftell again,
	//    and subtract "here" from that value to find out how many bytes the data portion of
	//    the file is. then fseek back to "here" to go back to the original location.
	// 6. if that length is not an even multiple of FILE_FOOD_LEN, use `fatal` to say
	//    "file doesn't appear to be a food database (bad length)."
	// 7. return the open FILE*.

	return NULL;
}

// Read a single food item from the database into a struct. Returns true if it was able
// to read the whole food item, false if not.
bool read_food(FILE* db, Food* dest) {
	// You will use fread() here to read the four fields of dest in this order:
	//  1. dest->name
	//  2. dest->dept
	//  3. dest->price
	//  4. dest->stock

	// use &dest->whatever as the first argument to fread, and sizeof(dest->whatever) as the
	// second. the third is just 1, and the last is db.

	// However, fread() can fail. It returns how many "things" it read - since we told it to
	// read 1 "thing" with the third argument, it will return 1 if successful, and 0 if not.

	// So: if *any* of the fread() calls fail (return 0), return false.
	// otherwise, return true.

	return false;
}

// Prints all foods in the database, regardless of what file position you are at.
void print_all_foods(FILE* db) {
	// 1. Seek to offset 8 in db (this is after the magic and endianness.)
	// 2. Make a Food variable. Not a Food*, a Food.
	// 3. While read_food into that variable's address is true...
	//    4. Use print_food to print it out.
}

// Prints all foods in a particular department, regardless of what file position you are at.
// Prints a message and exits if deptStr is not a valid department.
void print_foods_in_department(FILE* db, const char* deptStr) {
	// Very similar to print_all_foods. Two differences:
	// 1. first use str_to_department to convert deptStr to a Department.
	//    - if it returns Invalid, use fatal() to say "invalid department name!"
	// 2. inside the loop, only print the foods whose dept field is equal to the department
	//    that you converted deptStr into.
}

void add_food(FILE* db,
	const char* deptStr, const char* name, const char* priceStr, const char* stockStr) {

	// First, declare a variable like:
	//   Food new_food = {};
	// this creates a new food struct variable and the = {} initializes the fields to 0s.

	// Then, this function is gonna be mostly correctness checking. You need to:
	// - use str_to_department to convert deptStr into new_food.dept, and check it like you
	//   did in print_foods_in_department
	// - use sscanf to parse a double ("%lf") out of priceStr into new_food.price
	//   - and if that sscanf returned 0, use fatal() to say "price is not formatted correctly!"
	// - if the price is less than 0.01, say "invalid price!"
	// - use sscanf to parse an int out of stockStr into new new_food.stock
	//   - and if that sscanf returned 0, use fatal() to say "stock is not formatted correctly!"
	// - if the stock CASTED TO AN INT is less than 0, say "invalid stock!"
	// - if the length of name is 0, say "item name too short!"
	// - if the length of name is too big to fit into new_food.name, say "item name too long!"

	// then, once all those checks succeed:
	// - use strcpy to copy name into new_food.name.
	// - seek to the end of the database.
	// - write the four fields of new_food individually, similarly to how you read them
	//   in read_food.
}

int main(int argc, char** argv) {
	if(argc == 3 && streq(argv[1], "new")) {
		new_database(argv[2]);
	} else if(argc == 3 && streq(argv[1], "print")) {
		FILE* db = open_database(argv[2]);
		print_all_foods(db);
		fclose(db);
	} else if(argc == 4 && streq(argv[1], "printdept")) {
		FILE* db = open_database(argv[2]);
		print_foods_in_department(db, argv[3]);
		fclose(db);
	} else if(argc == 7 && streq(argv[1], "add")) {
		FILE* db = open_database(argv[2]);
		add_food(db, argv[3], argv[4], argv[5], argv[6]);
		fclose(db);
	} else {
		printf("How to use this program:\n");
		printf("  %s new filename\n", argv[0]);
		printf("      creates a new empty food database named filename.\n");
		printf("  %s print filename\n", argv[0]);
		printf("      prints all foods in filename.\n");
		printf("  %s printdept filename deptname\n", argv[0]);
		printf("      prints only foods from the deptname department in filename.\n");
		printf("  %s add filename deptname name price stock\n", argv[0]);
		printf("      adds a new food to filename.\n");
	}

    return 0;
}